from collections import deque, defaultdict

def bfs(start, max_distance, graph, visited):
    queue = deque([start])
    visited[start] = True
    distance = 0
    while queue and distance < max_distance:
        distance += 1
        for _ in range(len(queue)):
            current = queue.popleft()
            for neighbor in graph[current]:
                if not visited[neighbor]:
                    visited[neighbor] = True
                    queue.append(neighbor)

def main():
    import sys
    input = sys.stdin.read
    data = input().splitlines()
    
    n, m, k = map(int, data[0].split())
    
    # Построение графа
    graph = defaultdict(list)
    
    for i in range(1, m + 1):
        a, b = map(int, data[i].split())
        graph[a].append(b)
        graph[b].append(a)
    
    # Множество для охраняемых складов
    guarded = set()
    
    # Обход для каждого охранника
    for i in range(m + 1, m + k + 1):
        p, h = map(int, data[i].split())
        visited = [False] * (n + 1)  # Массив для отслеживания посещенных складов
        bfs(p, h, graph, visited)
        
        # Добавляем охраняемые склады в множество
        for j in range(1, n + 1):
            if visited[j]:
                guarded.add(j)
    
    # Вывод результатов
    guarded_list = sorted(guarded)
    print(len(guarded_list))
    if guarded_list:
        print(' '.join(map(str, guarded_list)))
    else:
        print()

if __name__ == "__main__":
    main()